
# q-expansion of newform 528.6.a.j, downloaded from the LMFDB on 11 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 528
weight = 6
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [9], [50], [-2], [121], [966], [1964], [-1246], [-136], [-7824], [4752], [4650], [7536], [14582], [-3984], [12350], [22380], [-15662], [29564], [-55536], [-63258], [40606], [-81808], [116434], [20734], [-47464], [106524], [74192], [192846], [-174442], [-34554], [143612], [136142], [292114], [386444], [143030], [-138778], [-423376], [519948], [62456], [319876], [1798], [-407688], [-806006], [-140384], [-800696], [954442], [1097316], [-604488], [781298], [-421568], [-399548], [-696290], [955300], [1000130], [-225300], [1317086], [341098], [-1622506], [-1543696], [-2258578], [2897884], [-2222950], [2082968], [-190354], [-282078], [-3280616], [-1929618], [-3805856], [-3862374], [1127034], [3746952], [1094932], [-3925430], [-3831920], [1902664], [93570], [-4890974], [-5206370], [-2359754], [1890516], [2155910], [-6706260], [7706126], [-3837310], [-2665156], [6354986], [-4114250], [-4128748], [4243164], [348972], [-9880596], [-4649600], [-7784912], [-8127836], [5146348], [9083294], [7237782], [-3710954], [3392710], [1370038], [-3257412], [-1746148], [6700828], [-6628162], [-7703614], [3717036], [-6544920], [6467760], [-13068302], [7810526], [10911594], [5093990], [-16383584], [-12928744], [-4584774], [9464704], [2445544], [-10181466], [3214764], [13018878], [11760314], [-4580240], [-19821396], [-3674040], [-10635036], [10585026], [8178000], [2157408], [-1152514], [-20293658], [18079784], [-10146248], [5961206], [-15101292], [18263266], [25365150], [4176518], [1162890], [-7610012], [3711322], [-7536968], [36449452], [11451472], [18577954], [11415912], [-33059022], [26460284], [-36568360], [19403368], [6282258], [-15836450], [-37870132], [-38944224], [-42352240], [-28171488], [33548098], [10790070], [-17522822], [-6392688], [-23395692], [-20849892], [-43624834], [46617540], [17216534], [11117520], [41190304], [-10704566], [-102914], [-21524338], [-33093548], [30405170], [16879800], [5890214], [31007580], [20890174], [-18450386], [-66839394], [2202558], [20247986], [-58373032], [-6843420], [-5630826], [13027716], [-52514424], [27572198], [-8060794], [28652544], [28209174], [-49005940], [-76084390], [-3849000], [92523108], [-76488462], [-8138588], [65001266], [36984498], [-24940882], [-3412860], [75307224], [-2867924], [-52707322], [17656006], [64721094], [35184708], [28003538], [39957964], [4797476], [13797804], [65196316], [-83758710], [16170078], [60597584], [64408452], [47139792], [93799790], [769922], [-80808392], [-85645552], [-12110666], [-72025034], [98588078], [78591886], [93887492], [-105772700], [115946394], [57710974], [2563560], [-123023782], [107593656], [-82615966], [123788642], [-71621114], [9244], [838252], [-146730072], [114591914], [39188244], [-9313404], [-46296280], [155489892], [41326414], [73649648], [-70017130], [-18824512], [-75722624], [161985600], [9616676], [-161065586], [106801192], [19711702], [-85477944], [145099088], [-178090890], [123042044], [62605272], [2310458], [105189362], [-128644782], [60289554], [-125124934], [11736948], [-88400994], [-49903926], [-161452158], [-142026956], [61610686], [115296030], [-146106174], [-8650480], [-147683230], [-27692232], [239039554], [-69096022], [82357750], [-98665760], [180733892], [-107354710], [4711354], [223205620], [-130921260], [-55623760], [47780568], [-2007118], [116798742], [-151446696], [-67223186], [42827904], [62227248], [-84223760], [102475202], [-123255204], [-61461336], [64360596], [-309782966], [19962548], [-286507984], [105054518], [-117739920], [-258195590], [10800650], [-114920372], [32489366], [-326172588], [-106119868], [-15418962], [-200083980], [-48056626], [-289760520], [-94688014], [-207333924], [214220994], [-70209528], [-261716196], [298639652], [123462070], [-70895660], [-95737952], [27785802], [-37283940], [178519146], [-133557062], [-274097108], [-315198472], [27316384], [370592834], [-256177980], [51349588], [-76405044], [14382448], [-176865114], [-381054462], [353530006], [-12266624], [-466724442], [-164299644], [-145261230], [153390608], [-111254934], [389683574], [281776666], [8425578], [142952836], [201466724], [-262317186], [457675112], [-111051338], [64459480], [366818416], [-363999842], [227164090], [229941386], [-346820514], [417962958], [346963890], [225759508], [-114047912], [152753080], [53687268], [231786248], [-118492606], [544042744], [381898280], [-428589176], [-212774156], [263007866], [509683340], [-236106978], [67996634], [-63854244], [29162688], [-47334072], [452477064], [184107786], [87307790], [-355710836], [-76180564], [-199368046], [678232444], [452874350], [455337146], [183531834], [-50517602], [228233540], [-258787298], [-96524520], [438847520], [575609962], [104908086], [713225472], [69678382], [267961494], [5184092], [-241169508], [107459584], [-230251546], [360929694], [-258532358], [-730049880], [271022520], [-754741430], [119008998], [-334756498], [116944018], [413459476], [12898078], [-140231182], [53061144], [414706364], [335560916], [-603313222], [-213879068], [-247599876], [-767161842], [-165010250], [85784738], [-487798652], [-281764420], [-644044966], [-382352556], [551748822], [-523237398], [803951248], [134255136], [-121106710], [129797094], [168739740], [-882259360], [-376191688], [-201384972]]
